package betterwithmods.common.blocks.mechanical;

import betterwithmods.BWMod;
import betterwithmods.api.block.IOverpower;
import betterwithmods.common.BWMBlocks;
import betterwithmods.common.BWRegistry;
import betterwithmods.common.blocks.BWMBlock;
import betterwithmods.common.blocks.BlockAesthetic;
import betterwithmods.common.blocks.mechanical.tile.TileSaw;
import betterwithmods.common.damagesource.BWDamageSource;
import betterwithmods.module.gameplay.MechanicalBreakage;
import betterwithmods.util.DirUtils;
import betterwithmods.util.InvUtils;
import net.minecraft.block.Block;
import net.minecraft.block.BlockLiquid;
import net.minecraft.block.SoundType;
import net.minecraft.block.material.Material;
import net.minecraft.block.state.BlockFaceShape;
import net.minecraft.block.state.BlockStateContainer;
import net.minecraft.block.state.IBlockState;
import net.minecraft.entity.Entity;
import net.minecraft.entity.EntityLivingBase;
import net.minecraft.init.SoundEvents;
import net.minecraft.item.ItemStack;
import net.minecraft.tileentity.TileEntity;
import net.minecraft.util.*;
import net.minecraft.util.math.AxisAlignedBB;
import net.minecraft.util.math.BlockPos;
import net.minecraft.world.IBlockAccess;
import net.minecraft.world.World;
import net.minecraftforge.fluids.IFluidBlock;
import net.minecraftforge.fml.relauncher.Side;
import net.minecraftforge.fml.relauncher.SideOnly;

import javax.annotation.Nullable;
import java.util.List;
import java.util.Optional;
import java.util.Random;

public class BlockSaw extends BWMBlock implements IBlockActive, IOverpower {
    private static final float HEIGHT = 0.71875F;
    private static final AxisAlignedBB D_AABB = new AxisAlignedBB(0.0F, 1.0F - HEIGHT, 0.0F, 1.0F, 1.0F, 1.0F);
    private static final AxisAlignedBB U_AABB = new AxisAlignedBB(0.0F, 0.0F, 0.0F, 1.0F, HEIGHT, 1.0F);
    private static final AxisAlignedBB N_AABB = new AxisAlignedBB(0.0F, 0.0F, 1.0F - HEIGHT, 1.0F, 1.0F, 1.0F);
    private static final AxisAlignedBB S_AABB = new AxisAlignedBB(0.0F, 0.0F, 0.0F, 1.0F, 1.0F, HEIGHT);
    private static final AxisAlignedBB W_AABB = new AxisAlignedBB(1.0F - HEIGHT, 0.0F, 0.0F, 1.0F, 1.0F, 1.0F);
    private static final AxisAlignedBB E_AABB = new AxisAlignedBB(0.0F, 0.0F, 0.0F, HEIGHT, 1.0F, 1.0F);

    public BlockSaw() {
        super(Material.field_151575_d);
        this.func_149711_c(2.0F);
        this.func_149672_a(SoundType.field_185848_a);
        this.func_180632_j(this.field_176227_L.func_177621_b().func_177226_a(DirUtils.FACING, EnumFacing.UP));
    }


    @Override
    public boolean hasTileEntity(IBlockState state) {
        return true;
    }

    @Nullable
    @Override
    public TileEntity createTileEntity(World world, IBlockState state) {
        return new TileSaw();
    }

    @Override
    public int func_149738_a(World world) {
        return 10;
    }

    @Override
    public IBlockState getStateForPlacement(World world, BlockPos pos, EnumFacing facing, float flX, float flY, float flZ, int meta, EntityLivingBase placer, EnumHand hand) {
        IBlockState state = super.getStateForPlacement(world, pos, facing, flX, flY, flZ, meta, placer, hand);
        return setFacingInBlock(state, DirUtils.getOpposite(facing));
    }

    @Override
    public void func_180633_a(World world, BlockPos pos, IBlockState state, EntityLivingBase entity, ItemStack stack) {
        EnumFacing facing = DirUtils.convertEntityOrientationToFacing(entity, EnumFacing.NORTH);
        world.func_175656_a(pos, world.func_180495_p(pos).func_177226_a(DirUtils.FACING, facing));
    }

    @Override
    public void func_176213_c(World world, BlockPos pos, IBlockState state) {
        super.func_176213_c(world, pos, state);
        world.func_180497_b(pos, this, func_149738_a(world), 5);
    }

    @Override
    public boolean func_149662_c(IBlockState state) {
        return false;
    }

    @Override
    public boolean func_149686_d(IBlockState state) {
        return false;
    }

    @Override
    public AxisAlignedBB func_185496_a(IBlockState state, IBlockAccess world, BlockPos pos) {
        state = state.func_185899_b(world, pos);
        EnumFacing facing = getFacing(state);
        switch (facing) {
            case DOWN:
                return D_AABB;
            case UP:
                return U_AABB;
            case NORTH:
                return N_AABB;
            case SOUTH:
                return S_AABB;
            case WEST:
                return W_AABB;
            case EAST:
            default:
                return E_AABB;
        }
    }

    @Override
    public void func_189540_a(IBlockState state, World world, BlockPos pos, Block block, BlockPos other) {
        world.func_180497_b(pos, this, func_149738_a(world), 5);
    }

    @Override
    public void func_180650_b(World world, BlockPos pos, IBlockState state, Random rand) {
        withTile(world, pos).ifPresent(TileSaw::onChanged);
        if (isActive(state)) {
            if (!world.field_72995_K) {
                sawBlockInFront(world, pos, rand);
                world.func_180497_b(pos, this, func_149738_a(world) + rand.nextInt(6), 5);
            }
        }
    }

    @Override
    public void onChangeActive(World world, BlockPos pos, boolean newValue) {
        Random rand = world.field_73012_v;
        emitSawParticles(world, pos, rand, EnumParticleTypes.SMOKE_NORMAL, 5);
        if (newValue) {
            world.func_180497_b(pos, this, func_149738_a(world) + rand.nextInt(6), 5);
            world.func_184133_a(null, pos, SoundEvents.field_187782_ds, SoundCategory.BLOCKS, 1.0F + rand.nextFloat() * 0.1F, 1.5F + rand.nextFloat() * 0.1F);
        } else {
            world.func_184133_a(null, pos, SoundEvents.field_187782_ds, SoundCategory.BLOCKS, 1.0F + rand.nextFloat() * 0.1F, 0.75F + rand.nextFloat() * 0.1F);
        }
    }

    @Override
    public void func_180634_a(World world, BlockPos pos, IBlockState state, Entity entity) {
        if (isActive(state) && entity instanceof EntityLivingBase) {
            EnumFacing dir = getFacing(world, pos);

            DamageSource source = BWDamageSource.getSawDamage();

            int damage = 4;
            boolean unobstructed = true;
            for (int i = 0; i < 3; i++) {
                BlockPos pos2 = new BlockPos(pos.func_177958_n(), pos.func_177956_o() - i, pos.func_177952_p()).func_177972_a(dir);
                Block block = world.func_180495_p(pos2).func_177230_c();
                IBlockState blockState = world.func_180495_p(pos2);
                if (block == BWMBlocks.AESTHETIC && blockState.func_177229_b(BlockAesthetic.TYPE).getMeta() < 2) {
                    source = BWDamageSource.getChoppingBlockDamage();
                    damage *= 3;
                    if (blockState.func_177229_b(BlockAesthetic.TYPE).getMeta() == 0 && unobstructed)
                        world.func_175656_a(pos2, BWMBlocks.AESTHETIC.func_176223_P().func_177226_a(BlockAesthetic.TYPE, BlockAesthetic.EnumType.CHOPBLOCKBLOOD));
                    break;
                } else if (!world.func_175623_d(pos2) && !(block instanceof BlockLiquid) && !(block instanceof IFluidBlock))
                    break;
                else if (!world.func_175623_d(pos2))
                    unobstructed = false;
            }
            if (source != null && entity.func_70097_a(source, damage)) {
                ((EntityLivingBase) entity).field_70718_bc = 60;
                world.func_184133_a(null, pos, SoundEvents.field_187782_ds, SoundCategory.BLOCKS, 1.0F + world.field_73012_v.nextFloat() * 0.1F, 1.5F + world.field_73012_v.nextFloat() * 0.1F);
            }
        }
    }

    @Override
    public boolean isSideSolid(IBlockState state, IBlockAccess world, BlockPos pos, EnumFacing side) {
        return side != getFacing(state);
    }

    public EnumFacing getFacing(IBlockAccess world, BlockPos pos) {
        return getFacing(world.func_180495_p(pos));
    }


    public EnumFacing getFacing(IBlockState state) {
        return state.func_177229_b(DirUtils.FACING);
    }

    public IBlockState setFacingInBlock(IBlockState state, EnumFacing facing) {
        return state.func_177226_a(DirUtils.FACING, facing);
    }

    @Override
    public void overpower(World world, BlockPos pos) {
        if (MechanicalBreakage.saw)
            InvUtils.ejectBrokenItems(world, pos, new ResourceLocation(BWMod.MODID, "block/saw"));
        world.func_184133_a(null, pos, SoundEvents.field_187539_bB, SoundCategory.BLOCKS, 0.3F, world.field_73012_v.nextFloat() * 0.1F + 0.45F);
        world.func_175698_g(pos);
    }


    public void emitSawParticles(World world, BlockPos pos, Random rand, EnumParticleTypes type, int amount) {
        EnumFacing facing = getFacing(world, pos);
        float xPos = pos.func_177958_n();
        float yPos = pos.func_177956_o();
        float zPos = pos.func_177952_p();
        float xExtent = 0.0F;
        float zExtent = 0.0F;

        switch (facing) {
            case DOWN:
                xPos += 0.5F;
                zPos += 0.5F;
                xExtent = 1.0F;
                break;
            case UP:
                xPos += 0.5F;
                zPos += 0.5F;
                yPos += 1.0F;
                xExtent = 1.0F;
                break;
            case NORTH:
                xPos += 0.5F;
                yPos += 0.5F;
                xExtent = 1.0F;
                break;
            case SOUTH:
                xPos += 0.5F;
                yPos += 0.5F;
                zPos += 1.0F;
                xExtent = 1.0F;
                break;
            case WEST:
                yPos += 0.5F;
                zPos += 0.5F;
                zExtent = 1.0F;
                break;
            default:
                yPos += 0.5F;
                zPos += 0.5F;
                xPos += 1.0F;
                zExtent = 1.0F;
        }
        for (int i = 0; i < amount; i++) {
            float smokeX = xPos + (rand.nextFloat() - 0.5F) * xExtent;
            float smokeY = yPos + rand.nextFloat() * 0.1F;
            float smokeZ = zPos + (rand.nextFloat() - 0.5F) * zExtent;
            world.func_175688_a(type, smokeX, smokeY, smokeZ, 0.0D, 0.0D, 0.0D);
        }
    }

    private void sawBlockInFront(World world, BlockPos pos, Random rand) {
        BlockPos blockPos = pos.func_177972_a(getFacing(world, pos));
        BWRegistry.WOOD_SAW.craftRecipe(world, blockPos, rand, world.func_180495_p(blockPos));
    }

    @Override
    @SideOnly(Side.CLIENT)
    public void func_180655_c(IBlockState state, World world, BlockPos pos, Random rand) {
        if (isActive(state)) {
            emitSawParticles(world, pos, rand, EnumParticleTypes.SMOKE_NORMAL, 5);

            List<EntityLivingBase> entities = world.func_72872_a(EntityLivingBase.class, new AxisAlignedBB(pos, pos.func_177982_a(1, 1, 1)));
            if (!entities.isEmpty())
                emitSawParticles(world, pos, rand, EnumParticleTypes.REDSTONE, 20);
        }
    }

    @Override
    public IBlockState func_176203_a(int meta) {
        int active = meta & 1;
        int facing = meta >> 1;
        return this.func_176223_P().func_177226_a(ACTIVE, active == 1).func_177226_a(DirUtils.FACING, EnumFacing.func_82600_a(facing));
    }

    @Override
    public int func_176201_c(IBlockState state) {
        int active = state.func_177229_b(ACTIVE) ? 1 : 0;
        int facing = state.func_177229_b(DirUtils.FACING).func_176745_a() << 1;
        return active | facing;
    }

    @Override
    protected BlockStateContainer func_180661_e() {
        return new BlockStateContainer(this, ACTIVE, DirUtils.FACING);
    }

    public Optional<TileSaw> withTile(World world, BlockPos pos) {
        return Optional.ofNullable(getTile(world, pos));
    }

    public TileSaw getTile(World world, BlockPos pos) {
        TileEntity tile = world.func_175625_s(pos);
        if (tile instanceof TileSaw)
            return (TileSaw) tile;
        return null;
    }

    @Override
    public boolean canPlaceTorchOnTop(IBlockState state, IBlockAccess world, BlockPos pos) {
        return getFacing(state) != EnumFacing.UP;
    }

    @Override
    public BlockFaceShape func_193383_a(IBlockAccess worldIn, IBlockState state, BlockPos pos, EnumFacing face) {
        return face != getFacing(state).func_176734_d() ? BlockFaceShape.UNDEFINED : BlockFaceShape.SOLID;
    }

    @Override
    public boolean rotateBlock(World world, BlockPos pos, EnumFacing axis) {
        if (super.rotateBlock(world, pos, axis)) {
            setActive(world, pos, false);
            return true;
        }
        return false;
    }
}
